package bloques;

import java.util.*;
import interfaz.deshacer.*;
import javax.swing.undo.*;
import util.bloqueIndice;

/**
 * <p>Title: BloqueParalelo.java</p>
 * <p>Description: Clase para los bloques paralelos</p>
 * @author Diego Daz Garca
 * @version 1.0
 */

public class BloqueParalelo
    extends Bloque {

  static final long serialVersionUID = 1111135267852834065L;

  /**Constructor de la clase bloqueParalelo
   * @param nombreInicio Es el nombre del nuevo bloque
   * @param padre_ Bloque padre del bloque a crear*/
  public BloqueParalelo(String nombreInicio, Bloque padre_) {
    super(nombreInicio, padre_);
  }

  /**Calcula el mttf del bloque
   * @param reparable Indica la reparabilidad o no del bloque
   * @return El Mttf del bloque*/
  public double obtenerMttf(boolean reparable) {
    if (reparable == true) {
      return 1 / obtenerLambda(reparable);
    }

    double LambdaMedia = 0;
    double progresion;
    double M;
    double mttf;
    int i;

    Iterator e = componentes.iterator();
    while (e.hasNext()) {
      LambdaMedia += ( (Bloque) e.next()).obtenerLambda(reparable);
    }
    LambdaMedia = LambdaMedia / componentes.size();

    progresion = 0;
    for (i = 1; i < componentes.size() + 1; i++) {
      progresion += 1.0 / i;

    }
    M = progresion / LambdaMedia;

    mttf = M; // mttf es el mximo de M, 1/lambda1, 1/lambda2
    Iterator e2 = componentes.iterator();
    while (e2.hasNext()) {
      double aLambda = ( (Bloque) e2.next()).obtenerLambda(reparable);

      if (1.0 / aLambda > mttf) {
        mttf = 1.0 / aLambda;
      }
    }
    return mttf;
  }

  /**Calcula el mtbf del bloque
   * @param reparable Indica la reparabilidad o no del bloque
   * @return El Mtbf del bloque*/
  public double obtenerMtbf(boolean reparable) {
    return obtenerMttr() + obtenerMttf(reparable);
  }

  /**Calcula la tasa de fallos del bloque
   * @param reparable Indica la reparabilidad o no del bloque
   * @return La tasa de fallos del bloque*/
  public double obtenerLambda(boolean reparable) {
    if (reparable == false) {
      return (1.0 / obtenerMttf(reparable));
    }
    else {
      return (obtenerMu() * (1 - obtenerDisponibilidad())) /
          obtenerDisponibilidad();
    }
  }

  /**Calcula la fiabilidad del bloque. Hay que tener en cuenta que para
   * reparables la calcula mal (pone el mismo resultado que para no reparables)
   * @param reparable Indica la reparabilidad o no del bloque
   * @param t Valor del tiempo para el clculo de la fiabilidad
   * @return La fiabilidad del bloque*/
  public double obtenerFiabilidad(double t, boolean reparable) {
    double r = 1;
    Iterator i = componentes.iterator();
    while (i.hasNext()) {
      r *= (1 - ( (Bloque) i.next()).obtenerFiabilidad(t, reparable));
    }
    return (1 - r);
  }

  /**Calcula la disponibilidad del bloque. Hay que tener en cuenta que para
   * reparables la calcula mal (pone el mismo resultado que para no reparables)
   * @return La disponibilidad del bloque*/
  public double obtenerDisponibilidad() {
    double a = 1;
    Iterator i = componentes.iterator();
    while (i.hasNext()) {
      a *= (1 - ( (Bloque) i.next()).obtenerDisponibilidad());
    }
    return (1 - a);
  }

  /**Calcula la tasa de reparacin del bloque
   * @return La tasa de reparacin del bloque*/
  public double obtenerMu() {
    double sumatorioLambdas = 0;
    double sumatorioDenominador = 0;
    Iterator i = componentes.iterator();
    while (i.hasNext()) {
      Bloque bloqueActual = (Bloque) i.next();
      sumatorioLambdas += bloqueActual.obtenerLambda(true);
      sumatorioDenominador += bloqueActual.obtenerLambda(true)
          / bloqueActual.obtenerMu();
    }
    return (sumatorioLambdas / sumatorioDenominador);
  }

  /**Calcula el mttr del bloque
   * @return El Mttr del bloque*/
  public double obtenerMttr() {
    return (1.0 / obtenerMu());
  }

  /**Comprueba la validez para el clculo de un bloque
   * @param reparable Cierto si el bloque es reparable, falso en caso contrario
   * @return Un string que estar vaci si el diagrama est correcto para
   * realizar clculos y que en caso contrario contendr el motivo por el
   * cul no est correcto*/
  public String comprobar(boolean reparable) {
    String correcto = "";
    if (componentes == null) {
      correcto = "El bloque extensible " + nombre() + " est vaco";
    }
    if (componentes.size() < 1) {
      correcto = "El bloque extensible " + nombre() + " est vaco";
    }
    Iterator i = componentes.iterator();
    while (i.hasNext()) {
      correcto = ( (Bloque) i.next()).comprobar(reparable);
      if (correcto != "") {
        break;
      }
    }
    return correcto;
  }

  /**Crea un bloque simple hijo del bloque actual
   * @return El bloque creado*/
  public Bloque bloqueSimple() {
    Bloque nuevoBloque = new BloqueBasico("", this);
    componentes.add(nuevoBloque);
    int posicion = componentes.size() - 1;
    bloqueIndice bI = new bloqueIndice(nuevoBloque, posicion);
    setChanged();
    notifyObservers(bI);
    return nuevoBloque;
  }

  /**Crea un bloque expansible hijo del bloque actual
   * @param paralelo Cierto si el nuevo bloque es en paralelo, falso si es en
   * serie
   * @return El bloque creado*/
  public Bloque bloqueExpansible(boolean paralelo) {
    Bloque nuevoBloque;
    if (paralelo == true) {
      nuevoBloque = new BloqueParalelo("Paralelo", this);
    }
    else {
      nuevoBloque = new BloqueSerie("Serie", this);
    }
    componentes.add(nuevoBloque);
    int posicion = componentes.size() - 1;
    bloqueIndice bI = new bloqueIndice(nuevoBloque, posicion);
    setChanged();
    notifyObservers(bI);
    return nuevoBloque;
  }

  /**Elimina un bloque
   * @param posicion Posicin del bloque a eliminar dentro de la lista de hijos
   * del bloque padre
   * @return Cierto si se ha eliminado el bloque, falso en caso contrario*/
  public boolean eliminaBloque(int i) {
    componentes.remove(i);
    bloqueIndice bI = new bloqueIndice(null, i);
    setChanged();
    notifyObservers(bI);
    return true;
  }

  /**Aade un bloque a la lista de hijos
   * @param b Bloque a aadir
   * @param posicion Posicion que ocupa el bloque dentro de la lista de hijos*/
  public void add(Bloque b, int posicion) {
    b.asignarPadre(this);
    Iterator iterador = componentes.iterator();
    ArrayList nuevaListaBloques = new ArrayList();
    int i = 0;
    while (iterador.hasNext()) {
      if (i == posicion) {
        nuevaListaBloques.add(b);
      }
      nuevaListaBloques.add( (Bloque) iterador.next());
      i++;
    }
    // Si la posicion del bloque es mayor que su tamao se incluye ahora
    if (posicion == i) {
      nuevaListaBloques.add(b);
    }
    bloqueIndice bI = new bloqueIndice(b, posicion);
    componentes = nuevaListaBloques;
    setChanged();
    notifyObservers(bI);
  }

  /**Devuelve si un bloque es extensible o no
   * @return True si es extensible, false si no*/
  public boolean esExtensible() {
    return true;
  }

  /**Devuelve si un bloque es paralelo o no
   * @return True si es paralelo, false si no*/
  public boolean esParalelo() {
    return true;
  }
}
