package bloques;

import java.util.*;
import javax.swing.undo.*;
import interfaz.deshacer.*;
import util.bloqueIndice;

/**
 * <p>Title: BloqueSerie.java</p>
 * <p>Description: Clase para los bloques serie</p>
 * @author Diego Daz Garca
 * @version 1.0
 */

public class BloqueSerie
    extends Bloque {

  static final long serialVersionUID = 5922824709810074326L;

  /**Constructor de la clase bloqueSerie
   * @param nombreInicio Es el nombre del nuevo bloque
   * @param padre_ Bloque padre del bloque a crear*/
  public BloqueSerie(String nombreInicio, Bloque padre_) {
    super(nombreInicio, padre_);
  }

  /**Calcula el mttf del bloque
   * @param reparable Indica la reparabilidad o no del bloque
   * @return El Mttf del bloque*/
  public double obtenerMttf(boolean reparable) {
    return (1.0 / obtenerLambda(reparable));
  }

  /**Calcula la tasa de fallos del bloque
   * @param reparable Indica la reparabilidad o no del bloque
   * @return La tasa de fallos del bloque*/
  public double obtenerLambda(boolean reparable) {
    double lambda = 0;
    if (reparable == false) {
      Iterator i = componentes.iterator();
      while (i.hasNext()) {
        lambda += ( (Bloque) i.next()).obtenerLambda(reparable);
      }
    }
    else {
      lambda = (obtenerMu() * (1 - obtenerDisponibilidad())
                / obtenerDisponibilidad());
    }
    return lambda;
  }

  /**Calcula el mtbf del bloque
   * @param reparable Indica la reparabilidad o no del bloque
   * @return El Mtbf del bloque*/
  public double obtenerMtbf(boolean reparable) {
    return obtenerMttr() + obtenerMttf(reparable);
  }

  /**Calcula la tasa de reparacin del bloque
   * @return La tasa de reparacin del bloque*/
  public double obtenerMu() {
    double sumatorioLambdas = 0;
    double sumatorioDenominador = 0;
    Iterator i = componentes.iterator();

    while (i.hasNext()) {
      Bloque bloqueEvaluado = (Bloque) i.next();
      sumatorioLambdas += bloqueEvaluado.obtenerLambda(true);
      sumatorioDenominador += bloqueEvaluado.obtenerLambda(true)
          / bloqueEvaluado.obtenerMu();
    }
    return (sumatorioLambdas / sumatorioDenominador);
  }

  public double obtenerMttr() {
    return (1.0 / obtenerMu());
  }

  /**Calcula la fiabilidad del bloque
   * @param reparable Indica la reparabilidad o no del bloque
   * @param t Valor del tiempo para el clculo de la fiabilidad
   * @return La fiabilidad del bloque*/
  public double obtenerFiabilidad(double t, boolean reparable) {
    double r = 1;
    Iterator i = componentes.iterator();
    while (i.hasNext()) {
      r *= ( (Bloque) i.next()).obtenerFiabilidad(t, reparable);
    }
    return r;
  }

  /**Calcula la disponibilidad del bloque
   * @return La disponibilidad del bloque*/
  public double obtenerDisponibilidad() {
    double a = 1;
    Iterator i = componentes.iterator();
    while (i.hasNext()) {
      a *= ( (Bloque) i.next()).obtenerDisponibilidad();
    }
    return a;
  }

  /**Comprueba la validez para el clculo de un bloque
   * @param reparable Cierto si el bloque es reparable, falso en caso contrario
   * @return Un string que estar vaci si el diagrama est correcto para
   * realizar clculos y que en caso contrario contendr el motivo por el
   * cul no est correcto*/
  public String comprobar(boolean reparable) {
    String correcto = "";
    if (componentes == null) {
      correcto = "El bloque extensible " + nombre() + " est vaco";
    }
    if (componentes.size() < 1) {
      correcto = "El bloque extensible " + nombre() + " est vaco";
    }
    Iterator i = componentes.iterator();
    while (i.hasNext()) {
      correcto = ( (Bloque) i.next()).comprobar(reparable);
      if (correcto != "") {
        break;
      }
    }
    return correcto;
  }

  /**Crea un bloque simple hijo del bloque actual
   * @return El bloque creado*/
  public Bloque bloqueSimple() {
    Bloque nuevoBloque = new BloqueBasico("", this);
    componentes.add(nuevoBloque);
    int posicion = componentes.size() - 1;
    bloqueIndice bI = new bloqueIndice(nuevoBloque, posicion);
    setChanged();
    notifyObservers(bI);
    return nuevoBloque;
  }

  /**Crea un bloque expansible hijo del bloque actual
   * @param paralelo Cierto si el nuevo bloque es en paralelo, falso si es en
   * serie
   * @return El bloque creado*/
  public Bloque bloqueExpansible(boolean paralelo) {
    Bloque nuevoBloque;
    if (paralelo == true) {
      nuevoBloque = new BloqueParalelo("Paralelo", this);
    }
    else {
      nuevoBloque = new BloqueSerie("Serie", this);
    }
    componentes.add(nuevoBloque);
    int posicion = componentes.size() - 1;
    bloqueIndice bI = new bloqueIndice(nuevoBloque, posicion);
    setChanged();
    notifyObservers(bI);
    return nuevoBloque;
  }

  /**Elimina un bloque
   * @param posicion Posicin del bloque a eliminar dentro de la lista de hijos
   * del bloque padre
   * @return Cierto si se ha eliminado el bloque, falso en caso contrario*/
  public boolean eliminaBloque(int i) {
    componentes.remove(i);
    bloqueIndice bI = new bloqueIndice(null, i);
    setChanged();
    notifyObservers(bI);
    return true;
  }

  /**Aade un bloque a la lista de hijos
   * @param b Bloque a aadir
   * @param posicion Posicion que ocupa el bloque dentro de la lista de hijos*/
  public void add(Bloque b, int posicion) {
    b.asignarPadre(this);
    Iterator iterador = componentes.iterator();
    ArrayList nuevaListaBloques = new ArrayList();
    int i = 0;
    while (iterador.hasNext()) {
      if (i == posicion) {
        nuevaListaBloques.add(b);
      }
      nuevaListaBloques.add( (Bloque) iterador.next());
      i++;
    }
    // Si la posicion del bloque es mayor que su tamao se incluye ahora
    if (posicion == i) {
      nuevaListaBloques.add(b);
    }
    bloqueIndice bI = new bloqueIndice(b, posicion);
    componentes = nuevaListaBloques;
    setChanged();
    notifyObservers(bI);
  }

  /**Devuelve si un bloque es extensible o no
   * @return True si es extensible, false si no*/
  public boolean esExtensible() {
    return true;
  }
}
